#!perl -w
use Test::More tests => 12;
use strict;
use Cwd;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::Error  qw( GetError ExistsError );
use DE_EPAGES::Core::API::File   qw( CopyDirectory CreateDirectory WriteFile CompareDirectories );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (PackTar UnpackTar);

sub Main {
    my $TempDir = tempdir( CLEANUP => 1 );
    my ($Content, $aPackedFiles);

    my $Cwd = getcwd;

    foreach (qw(11 111 112 121 131 21 211 213 221 222)) {
        $Content = $_ x 1000;
        WriteFile(sprintf("$TempDir/%s.txt", join('/',split(//,$_))), \$Content, {create_directory=>1});
    }
    $Content = '1.1' x 1000;
    WriteFile("$TempDir/1/.1.txt", \$Content);
    CopyDirectory("$TempDir/1", "$TempDir/$_") foreach qw(11 12 a b);
    CreateDirectory("$TempDir/3");
    CopyDirectory("$TempDir/1", "$TempDir/$_") foreach qw(C F);

    $aPackedFiles = PackTar('../pack.tar', undef, undef, {Print => 1} );
    ok( -f '../pack.tar', 'PackTar(../pack.tar) creates ../pack.tar');
    ok('tar.t' ~~ @$aPackedFiles, '../pack.tar contains tar.t');

    $aPackedFiles = PackTar('../pack.tgz', undef, undef, {Print => 1} );
    ok( -f '../pack.tgz', 'PackTar(../pack.tgz) creates ../pack.tgz');
    ok('tar.t' ~~ @$aPackedFiles, '../pack.tgz contains tar.t');
    ok(-s '../pack.tgz' < -s '../pack.tar', '../pack.tgz is smaller than ../pack.tar');

    my $Success = 1;
    foreach my $File (@{PackTar('../pack.tar', '.', ['*.t'], {Print => 1})}) {
        $Success = 0 unless $File =~ /\.t$/
    }
    ok(-s '../pack.tar' && $Success, 'PackTar(../pack.tar,.,[*.t]) contains only .t files');

    my $Error;

    eval { PackTar('../pack.tar', 'blurb', undef, {Print => 1}); }; ##no critic qw(RequireExistsError)
    $Error = GetError();
    ok(ExistsError(), 'PackTar(../pack.tar, blurb) -> '. $Error->message);

    eval { PackTar('../pack.tar', $TempDir, ['*.blurb'], {Print => 1}); }; ##no critic qw(RequireExistsError)
    $Error = GetError();
    ok(ExistsError(), 'PackTar(../pack.tar, $TempDir, [*.blurb]) -> '. $Error->message);

    eval { PackTar('../pack.txt', $TempDir, undef, {Print => 1}); }; ##no critic qw(RequireExistsError)
    $Error = GetError();
    ok(ExistsError(), 'PackTar(../pack.txt, $TempDir) -> '. $Error->message);

    my $aPackedFiles2 = PackTar('../pack.tar', '/', [$TempDir], {Print => 1});
    $aPackedFiles = PackTar('../pack.tar', $TempDir, undef, {Print => 1});
    ok(scalar @$aPackedFiles +1 == scalar @$aPackedFiles2, 'PackTar($_, $TempDir) does not contain $TempDir; PackTar($_, /, [$TempDir]) does');

    $aPackedFiles = PackTar('../pack.tgz', $TempDir, undef, {Print => 1});
    $aPackedFiles2 = UnpackTar('../pack.tgz', "$TempDir/out1", {Print => 1});
    ok(scalar @$aPackedFiles == scalar @$aPackedFiles2, 'PackTar($_, $TempDir) eq UnpackTar(../pack.tgz, $TempDir/out1)');

    $aPackedFiles = UnpackTar('../pack.tar', "$TempDir/out2", {Print => 1});
    ok(CompareDirectories("$TempDir/out1", "$TempDir/out2"), 'CompareDirectories($TempDir/out1, $TempDir/out2)');

    chdir $Cwd;
    unlink('../pack.tar');
    unlink('../pack.tar.gz');
}

RunScript( Sub => \&Main );
1;
