#========================================================================================
# package      DE_EPAGES::DHLIntaship::t::ShippingLabel
# state        private
#----------------------------------------------------------------------------------------
# description  test the Getter of a shipping label
#========================================================================================
package TestShippingLabel;

use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use Test::More;
use Test::Exception;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            PositiveCase();
            NegativeCase();
            # tests
            done_testing();
        }
    );
}

sub PositiveCase {
    my $subTestName = (caller(0))[3];
    my $ShippingLabel = StubShippingLabelDefault();
    my $DHLIntrashipServices = StubServices();
    my $ProductData = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $ShippingLabel, $DHLIntrashipServices );
    $ProductData->isComplete();

    is($ProductData->{'Alias'},'ShippingLabelAlias' ,"$subTestName - get the Alias - 11");
    is($ProductData->{'EstimatedDeliveryDate'},'2222-02-22' ,"$subTestName - get the EstimatedDeliveryDate - 2222-02-22");
    is($ProductData->{'DHLServices'},'getServicesFromDatabaseResponse' ,"$subTestName - get the DHLServices - getServicesFromDatabaseResponse");
    isa_ok($ProductData->{'ShippingFulfillmentMethod'}, 'StubShippingLabelDefault');
    is(scalar @{$ProductData->getStatesAsWarning()},0 , "$subTestName - empty array of warnings - 0")
}

sub NegativeCase {
    my $subTestName = (caller(0))[3];
    my $ShippingLabel = StubShippingLabelEmpty();
    my $DHLIntrashipServices = StubServices();
    my $ProductData = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $ShippingLabel, $DHLIntrashipServices );
    throws_ok(
       sub{$ProductData->isComplete()}
       ,qr/Message=ShippingFulfillmentMethod is/,
       "$subTestName - checks the Error case for wrong inputtype - WrongInputType"
    );
    is(scalar @{$ProductData->getStatesAsWarning()},2 , "$subTestName - get a list of warnings - 3");
}

sub StubShippingLabelDefault {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
        package StubShippingLabelDefault;
        use DateTime;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            my $dt = DateTime->new(
                year       => 2222,
                month      => 2,
                day        => 22,
                hour       => 22,
                minute     => 22,
            );
            return $dt if $AttributeName eq 'EstimatedDeliveryDate';
            return $self if $AttributeName eq 'ShippingFulfillmentMethod';
            return '012345678901' if $AttributeName eq 'CustomerNumber';
        };
        sub alias { return 'ShippingLabelAlias'; };
        return bless {}, 'StubShippingLabelDefault';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
    }
}

sub StubShippingLabelEmpty {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
        package StubShippingLabelEmpty;
        sub get{
            my $self = shift;
            my ($AttributeName) = @_;
            return undef if $AttributeName eq 'EstimatedDeliveryDate';
            return undef if $AttributeName eq 'ShippingFulfillmentMethod';
        };
        sub alias { return undef; };
        return bless {}, 'StubShippingLabelEmpty';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages ProhibitMagicNumbers )
    }
}
sub StubServices {
    {
        ## no critic ( ProhibitNestedSubs ProhibitMultiplePackages )
        package StubServices;
        sub getServicesFromDatabase{ return 'getServicesFromDatabaseResponse'; };
        return bless {}, 'StubServices';
        ## use critic ( ProhibitNestedSubs ProhibitMultiplePackages )
    }
}
RunScript( Sub => \&Main );

1;