/*
 * @copyright   © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module      de_epages.externalcontent.ui.contentslider
 *
 * @revision    $$
 */
/*jslint nomen: true*/
/*global define*/
define('de_epages/externalcontent/ui/contentslider', [
    'jquery/ui/widget',
    'de_epages',
    '$tmpl!./contentslider',

    'jquery/cycle'
], function ($, de_epages, template) {
    'use strict';
    // Constants.
    var gClassNames = 'contentslider';

    // The actual widget.
    $.widget('ui.externalcontentUiContentslider', {

        options: {
            effect: 'fade',
            arrows: false,
            shadow: true,
            border: 'transparent',
            automatic: true,
            displaytime: 3,
            cycleOptions: {},
            slides: []
        },

        _create: function () {
            var self = this,
                o = self.options,
                $firstImage,
                zIndexOfSlides,
                images;

            if (o.slides.length === 0) {
                return self.destroy();
            }

            // Render template. Leave *src* attribute of *img* tags out for now.
            // We want to start loading the images later.
            $.extend(self, $.tmplItem(template(o), 'elements'));

            self.element
                .empty() // Delete &nbsp; which may have been added to the *self.element* by *CKEditor*.
                .append(self.list, self.pager, self.arrows) // Append template items to *self.element*.
                .addClass(gClassNames) // Add gadget specific css classes.
                .css('border-color', o.border); // Set *border-color*.
            if (o.shadow) {
                self.element.addClass('contentslider-shadow');
            }

            $firstImage = self.list.children('li').first().children();

            if ($firstImage.is('a')) {
                $firstImage = $firstImage.children('img').first();
            }

            // *create* cycle, when *$firstImage* is loaded.
            $firstImage.one('load', function (event) {
                var firstLi = $(this).closest('li'),
                    size = {
                        width: firstLi.width(),
                        height: firstLi.height()
                    };

                // Set size of all list items.
                // This way we do not have to wait
                // for all images to be loaded. Great for slow ol' firefox :)
                self.list.css(size).children('li').css(size);

                if (o.slides.length > 1) {
                    // Otherwise the *cycle* plugin won't work anyway.
                    self._createCycle();
                }
            });


            // Set *z-index* of control elements.
            zIndexOfSlides = o.slides.length;
            self.pager.css('z-index', zIndexOfSlides + 20);
            if (o.arrows && self.next && self.prev) {
                self.next.css('z-index', zIndexOfSlides + 10);
                self.prev.css('z-index', zIndexOfSlides + 10);
            }

            // Set *src* of *img* elements.
            images = self.list.find('img');
            $.each(o.slides, function (index, slide) {
                images[index].src = slide.fullpath;
            });
        },

        _createCycle: function () {
            var self = this,
                o = self.options;

            self.list.cycle($.extend({}, o.cycleOptions, {
                timeout: o.automatic ? (1000 * o.displaytime) : 0,
                pager: self.pager,
                fx: o.effect
            }, o.arrows ? {
                next: self.next,
                prev: self.prev
            } : {}));
        },

        destroy: function () {
            var self = this;

            // Remove all traces of the widget.
            if (self.list) {
                self.list
                    .cycle('pause') // Otherwise *before* and *after* callbacks are still getting called for some reason.
                    .cycle('destroy');
            }

            self.element.empty().removeClass(gClassNames).removeClass('contentslider-shadow');

            return self._superApply(arguments);
        }
    });

    return de_epages;

});