define("ckeditor/plugins/eptextaligngroup/plugin", [
	"jquery",
	"ckeditor/ckeditor",
	"$dict!ep/dict"
], function ($, CKEDITOR, epDict) {

	/*
	 * @dictionary		ep.dict
	 *
	 * @translation		{TextAlignGroup}
	 */

	var currentClassName,
		alignments = ['left', 'center', 'right', 'block'],

		tTextAlignGroup = epDict.translate('TextAlignGroup');

	CKEDITOR.plugins.add( 'eptextaligngroup',
	{
		requires : [ 'menubutton', 'justify' ],

		init: function( editor ){
			var menuGroup = 'textaligngroup',
				uiMenuItems = {};

			// add menu group
			editor.addMenuGroup( menuGroup );

			// create menu entries different alignments
			for(var i = 0, len = alignments.length; i <len; i++){
				uiMenuItems['justify'+alignments[i]] = {
					label : editor.lang.justify[alignments[i]],
					command : 'justify'+alignments[i],
					group : menuGroup
				};
			}

			// add items to the menu
			editor.addMenuItems( uiMenuItems );

			// create menu button
			editor.ui.add( 'eptextaligngroup', CKEDITOR.UI_MENUBUTTON,{
				label : tTextAlignGroup,
				title : tTextAlignGroup,
				modes : { wysiwyg : 1 },
				className : 'cke_off cke_button_justifyleft',
				onRender: function(){
					var self = this;

					currentClassName = self.className;

					// selection handling --> show current alignment
					editor.on( 'selectionChange', function( ev ){
						var element = editor.getSelection().getStartElement(),
							button = $('#'+self._.id),
							align,
							newClassName;

						while(!element.getStyle('text-align')){
							var parent = element.getParent();

							if ( !parent ){
								break;
							}
							element = parent;
						}

						align = element.getStyle( 'text-align' );

						if(!align){
							align = 'left';
						}
						else if(align === 'justify'){
							align = 'block';
						}

						// set highlight state if text-align is set for current selection
						//self.setState(align ? CKEDITOR.TRISTATE_ON : CKEDITOR.TRISTATE_OFF);

						newClassName = 'cke_button_justify' + align;
						button.removeClass(currentClassName);
						button.addClass(newClassName);
						currentClassName = newClassName;
					});
				},
				onMenu : function(){
					return {
						justifyleft  : CKEDITOR.TRISTATE_OFF,
						justifycenter : CKEDITOR.TRISTATE_OFF,
						justifyright : CKEDITOR.TRISTATE_OFF,
						justifyblock : CKEDITOR.TRISTATE_OFF
					};
				}
			});
		}
	});

	return CKEDITOR;

});