/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.openWindow
 *
 * @revision		$Revision: 1.12 $
 */


define("ep/openwindow", [
	"jquery",
	"ep",
	"$dict!ep/dict",
	"$tmpl!ep/openwindow-mobilepreview",

	"ep/uri",
	"ep/ui/dialog",
	"ep/fn/busy"
], function ($, ep, epDict, tmplOpenwindowMobilepreview) {

	var dialogNode,
		iframeNode,
		lastDialog;

/*
 * @dictionary		ep.dict
 *
 * @translation		{Close}
 *								{GeneralSettings}
 *								{TooltipTextForMobileSettings}
 *								{useMobileTextEnabled}
 *								{useMobileTextDisabled}
 */

	ep.extend({ // open new window with predefined features
		destroyWindow: function(){
			if(dialogNode){
				ep(dialogNode).uiDialog('destroy');
				dialogNode = undefined;
			}
		},
		openMobileWindow: function(url, features, previewAdds){
			var mFeatures = $.extend({
					//ShopID: #Shop.ID,
					isMobilePreview: true,	// flag for mobile dialog
					showInfo: true,					// flag to decide if an info element should be shown in the mobile dialog
					width: 330,							// width of dialog
					height: 651,						// height of dialog
					iframe: {
						w: 285,								// width for embeded iframe
						h: 430								// height for embeded iframe
					},
					tooltipText: epDict.translate('TooltipTextForMobileSettings'),
					GeneralSettings: epDict.translate('GeneralSettings'),
					useMobileTextDisabled: epDict.translate('useMobileTextDisabled'),
					useMobileTextEnabled: epDict.translate('useMobileTextEnabled'),
					UseMobileSF: true
				}, features),
				mAdditionals = $.extend({
					dialogClass: 'ep-preview-mobile-storefront'
				}, previewAdds);

				ep.openWindow(url, mFeatures, mAdditionals);
		},
		openWindow: function( url, features, previewAdds ){
			// if function was called by openMobileWindow
			if(ep.openWindow.caller === ep.openMobileWindow){
				// if already a dialog exists and it isn't a mobile dialog (1)
				if(lastDialog && lastDialog!==1){
					ep.destroyWindow();
				}
				// mark last dialog as mobile dialog
				lastDialog = 1;
			}else{
				// if already a dialog exists and it isn't a standard dialog (2)
				if(lastDialog && lastDialog!==2){
					ep.destroyWindow();
				}
				// mark last dialog as standard dialog
				lastDialog = 2;
			}

			var $win = $(window),
				o = $.extend({
				//iframe: undefined,
				//ShopID: undefined,					// shop id (used to link to general settings)
				//tooltipText: undefined,			// (string) tooltip text to point to mobile settings
				//GeneralSettings: undefined,	// (string) translation for general settings
					isMobilePreview: false,
					width: $win.width()-30,
					height: $win.height()-40,
					externalUrl: false
				}, features),
				additional = {};							// object for additional dialog options

			if(previewAdds !== undefined){
				$.extend(additional, previewAdds);
			}

			if(o.width > $win.width()){
				o.position = 'right';
			}
			else{
				o.position = 'center';
			}

			// get index of hash
			var index = url.lastIndexOf("#"),
				hash = "";

			// if hash exists
			if(index !== -1){
				hash = url.substring(index);
				url = url.substring(0, index);
			}

			if(!dialogNode){
				dialogNode = 	ep('<div>');
				iframeNode = $('<iframe>')
					.appendTo(dialogNode)
					.addClass('epOpenWindowIframe')
					.css({
						border: 'none',
						width: o.isMobilePreview ? (o.iframe.w+'px') : '100%',
						height: o.isMobilePreview ? (o.iframe.h+'px') : '99%'
					});

				// additional contents for storefront preview
				if(o.isMobilePreview){
					var elems = {};

					// initialize template for tooltip/info and close button and fill it with data
					$.extend(
						elems,
						tmplOpenwindowMobilepreview($.extend(o, {Close: epDict.translate( 'Close' )}))
							.appendTo(dialogNode)
							.tmplItem('elements')
					);

					elems.closeButton.on("click", function(){
						dialogNode.uiDialog('close');
					});
				}

				dialogNode
					.uiDialog($.extend({
						maxWidth: o.width,
						width: o.width,
						height: o.height,
						resizable: false,
						modal:	true,
						position: o.position,
						buttons: [
							{text: epDict.translate( 'Close' ), click: function(){ ep(this).uiDialog('close'); }}
						]
					}, additional ))
					.busy();
				iframeNode.on('load',function(){
					dialogNode.busy('hide');

					if(o.externalUrl){
						dialogNode.uiDialog('option','title',url);
					}else{
						if(iframeNode.length && 	iframeNode[0].contentDocument){
							try{
								var titleText = iframeNode[0].contentDocument.title;
								if(titleText){
									dialogNode.uiDialog('option','title',titleText);
								}
							}
							catch(e){
								console.warn('unable to read title from iframe');
							}
						}
					}
				});
			} else{
				dialogNode
					.busy()
					.uiDialog('open')
					.uiDialog('option',{
						'height': o.height,
						'width': o.width,
						'position': o.position
					});
			}

			// set source attribute of iframe to load URL
			iframeNode.attr('src', url + (/\?/.test(url) ? "&" : "?") + "_=" + $.now() + hash );
		}
	});

	return ep;

});