define(['testsuite/qunit', 'jquery', 'ep/ui/core', '$ready!'], function (QUnit, $, ep) {

	QUnit.module("ep.ui.core");

	QUnit.asyncTest("ep.ui.imgData()", function() {

		var i1 = ep.ui.imgData('#images1 img'),
			i2 = ep.ui.imgData('#images2 img'),
			i3 = ep.ui.imgData( $('#images2 li img') ),
			i4 = ep.ui.imgData( 'ep/ui/core-test.jpg' ),
			i5 = ep.ui.imgData( {desc:'Object 1', src:'ep/ui/core-test.jpg'} ),
			i6 = ep.ui.imgData( [ 'ep/ui/core-test.jpg', {desc:'Object 2', src:'ep/ui/core-test.jpg'} ] ),

			e1 = false;

		QUnit.equal(
			i1.length,
			5,
			"ep.ui.imgData('#images1 img').length"
		);

		QUnit.equal(
			i2.length,
			2,
			"ep.ui.imgData('#images2 img').length"
		);

		i1.each(function( i, item ){
			QUnit.equal(
				item.desc,
				"Logo "+(i+1),
				"ep.ui.imgData('#images1 img').each( ... item.desc ... )"
			);
		});

		i2.each(function( i, item ){
			QUnit.equal(
				item.desc,
				"Title "+(i+1),
				"ep.ui.imgData('#images2 img').each( ... item.desc ... )"
			);
		});

		i3.each(function( i, item ){
			QUnit.equal(
				item.desc,
				"Title "+(i+1),
				"ep.ui.imgData( $('#images2 li img') ).each( ... item.desc ... )"
			);
		});

		i4.each(function( i, item ){
			QUnit.equal(
				item.src,
				'ep/ui/core-test.jpg',
				"ep.ui.imgData( 'ep/ui/core-test.jpg' ).each( ... item.src ... )"
			);
			QUnit.equal(
				item.desc,
				'',
				"ep.ui.imgData( 'ep/ui/core-test.jpg' ).each( ... item.desc ... )"
			);
		});

		i5.each(function( i, item ){
			QUnit.equal(
				item.src,
				'ep/ui/core-test.jpg',
				"ep.ui.imgData( {...} ).each( ... item.src ... )"
			);
			QUnit.equal(
				item.desc,
				'Object 1',
				"ep.ui.imgData( {...} ).each( ... item.desc ... )"
			);
		});

		i6.each(function( i, item ){
			QUnit.equal(
				(item.desc || 'Object 1'),
				'Object '+(i+1),
				"ep.ui.imgData( {...} ).each( ... item.desc ... )"
			);
		});

		QUnit.equal(
			typeof i1.current(),
			'object',
			"typeof ep.ui.imgData('#images1 img').current()"
		);

		QUnit.equal(
			i1.current().desc,
			'Logo 1',
			"ep.ui.imgData('#images1 img').current().desc"
		);

		QUnit.equal(
			i1.current(2).current().desc,
			'Logo 3',
			"ep.ui.imgData('#images1 img').current(2).current().desc"
		);

		QUnit.equal(
			i1.current(10).current().desc,
			'Logo 3',
			"ep.ui.imgData('#images1 img').current(10).current().desc"
		);

		QUnit.equal(
			i1.current('prev').current().desc,
			'Logo 2',
			"ep.ui.imgData('#images1 img').current('prev').current().desc"
		);

		QUnit.equal(
			i1.current('next').current().desc,
			'Logo 3',
			"ep.ui.imgData('#images1 img').current('next').current().desc"
		);

		QUnit.equal(
			i1.next().desc,
			'Logo 4',
			"ep.ui.imgData('#images1 img').next().desc"
		);

		QUnit.equal(
			i1.prev().desc,
			'Logo 2',
			"ep.ui.imgData('#images1 img').prev().desc"
		);

		QUnit.equal(
			i1.first().desc,
			'Logo 1',
			"ep.ui.imgData('#images1 img').first().desc"
		);

		QUnit.equal(
			i1.last().desc,
			'Logo 5',
			"ep.ui.imgData('#images1 img').last().desc"
		);

		i1.on('change',function(){
			e1 = !e1;
		});
		i1.current(1);

		QUnit.equal(
			e1,
			true,
			"ep.ui.imgData('#images1 img').on('change', toggle(e1) ).current(1); e1"
		);

		i1.off('change');
		i1.current(3);

		QUnit.equal(
			e1,
			true,
			"ep.ui.imgData('#images1 img').off('change').current(3); e1"
		);

		i1.on('resize',function(){
			e1 = !e1;
		});

		i1.on('resizestop',function(){
			e1 = !e1;
		});

		$(window).trigger('resize');

		QUnit.equal(
			e1,
			false,
			"ep.ui.imgData('#images1 img').on('resize', toggle(e1) ); .trigger(...); e1"
		);

		setTimeout(function(){

			QUnit.equal(
				e1,
				true,
				"ep.ui.imgData('#images1 img').on('resizestop', toggle(e1) ); .trigger(...); e1"
			);

			start();

		}, 200 );

	});

	QUnit.asyncTest("ep.ui.imgCenter()", function() {

		var img = $('#center1 img');

		ep.ui.imgCenter( img );

		QUnit.equal(
			parseInt(img.css('top')) >= -43 && img.css('top').substr(img.css('top').length-2,2) == 'px',
			1,
			"ep.ui.imgCenter( img ); img.css('top')"
		);

		QUnit.equal(
			img.css('left'),
			'-100px',
			"ep.ui.imgCenter( img ); img.css('left')"
		);

		ep.ui.imgCenter( img, true );

		QUnit.equal(
			img.css('top'),
			'0px',
			"ep.ui.imgCenter( img, true ); img.css('top')"
		);

		QUnit.equal(
			parseInt(img.css('left')) >= 83 && img.css('left').substr(img.css('left').length-2,2) == 'px',
			1,
			"ep.ui.imgCenter( img, true ); img.css('left')"
		);

		QUnit.equal(
			parseInt(img.css('width')) >= 632 && img.css('width').substr(img.css('width').length-2,2) == 'px',
			1,
			"ep.ui.imgCenter( img, true ); img.css('width')"
		);

		QUnit.equal(
			img.css('height'),
			'150px',
			"ep.ui.imgCenter( img, true ); img.css('height')"
		);

		start();
	});

	QUnit.asyncTest("ep.ui.changeOverFx()", function() {

		var container = $('#changeOver1'),
			from = container.children().eq(0),
			to = container.children().eq(1),
			duration = 3500,
			callback = $.noop,

			img = to.find('img');

		( ep.ui.changeOverFx("fade-centered") )( container, from, to, duration, callback );

		setTimeout(function(){

			QUnit.equal(
				from.find('img').is(':visible'),
				false,
				"ep.ui.changeOverFx( ... ); from.find('img').is(':visible')"
			);

			QUnit.equal(
				img.is(':visible'),
				true,
				"ep.ui.changeOverFx( ... ); to.find('img').is(':visible')"
			);

			QUnit.equal(
				parseInt(img.css('top')) >= 55 && img.css('top').substr(img.css('top').length-2,2) == 'px',
				1,
				"ep.ui.changeOverFx( ... ); to.find('img').css('top')"
			);

			QUnit.equal(
				img.css('left'),
				'0px',
				"ep.ui.changeOverFx( ... ); to.find('img').css('left')"
			);

			QUnit.equal(
				img.css('width'),
				'800px',
				"ep.ui.changeOverFx( ... ); to.find('img').css('width')"
			);

			QUnit.equal(
				parseInt(img.css('height')) >= 189 && img.css('height').substr(img.css('height').length-2,2) == 'px',
				1,
				"ep.ui.changeOverFx( ... ); to.find('img').css('height')"
			);

			start();
		}, 1200 );

	});

});