define(['testsuite/qunit', 'jquery', 'ep/ui/input', 'util/support', 'ep/ui/form', 'jquery/metaparse'], function (QUnit, $, ep, support) {

	QUnit.module("ep.ui.input");

	QUnit.asyncTest("ep.ui.input", function() {

		$('.ep-js').metaparse();

		var input = ep('input:first');

		equal(
			input.uiInput('Instance') instanceof $.Widget,
			true,
			"uiInput widget was created and instance of jQuery.Widget"
		);

		$.each(['_create', '_init', '_leave', '_stateChanged', '_tooltipInit', 'addClass', 'removeClass', '_change', '_changeAttr', '_setOption', 'destroy'], function (i, fn) {
			equal(
				$.isFunction($.ui.uiInput.prototype[fn]),
				true,
				'$.isFunction(' + fn + '):'
			);
		});

		// text input
		QUnit.equal(
			input.uiInput('option', 'placeholder'),
			'placeholder',
			"Get placeholder of text input"
		);

		input.uiInput('option', 'placeholder', 'new placeholder');

		QUnit.equal(
			input.uiInput('option', 'placeholder'),
			'new placeholder',
			"Get placeholder of text input after it was changed"
		);

		input.uiInput('addClass', 'myTestClass');

		equal(
			input.hasClass('myTestClass'),
			true,
			'text input has class "myTestClass"'
		);

		input.uiInput('removeClass', 'myTestClass');

		equal(
			input.hasClass('myTestClass'),
			false,
			'"myTestClass" was removed from text input'
		);

		input
			.trigger('focusin')
			.val('myTestString')
			.trigger('focusout');

		equal(
			input.val(),
			'myTestString',
			'Get value of text input after the value was changed'
		);

		input.uiInput('destroy');

		equal(
			input.data('epUiInput'),
			undefined,
			'uiInput widget was destroyed'
		);

		// radio
		QUnit.equal(
			$(':radio').length,
			2,
			"$(':radio').length"
		);

		QUnit.equal(
			$(':radio:checked').length,
			0,
			"$(':radio:checked').length"
		);

		$(':radio:first').trigger('click');
		QUnit.equal(
			$(':radio:checked').length,
			1,
			"$(':radio:checked').length"
		);

		// checkbox
		QUnit.equal(
			$(':checkbox').length,
			3,
			"$(':checkbox').length"
		);

		QUnit.equal(
			$(':checkbox:checked').length,
			0,
			"$(':checkbox:checked').length"
		);

		$(':checkbox:first').trigger('click');
		QUnit.equal(
			$(':checkbox:checked').length,
			1,
			"$(':checkbox:checked').length"
		);

		// button without form-attribute
		QUnit.equal(
			$('#buttonWithoutFormAttribute').data('events').click,
			undefined,
			"$('#buttonWithoutFormAttribute').data('events')['click']"
		);

		// button with form-attribute
		if ( support.inputFormAttr ){ // html5 browser
/*
			QUnit.equal(
				$('#buttonWithFormAttribute').data('events').click,
				undefined,
				"$('#buttonWithFormAttribute').data('events')['click']"
			);
*/
		} else { // non-html5 browser
			var clickObject = $('#buttonWithFormAttribute').data('events').click;

			QUnit.equal(
				clickObject && clickObject.length,
				1,
				"$('#buttonWithFormAttribute').data('events')['click']"
			);
		}

		QUnit.start();
	});

});