/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.slides
 *
 * @revision		$Revision: 1.2 $
 */

define("ep/ui/mobileslides", [
	"jquery",
	"ep",
	"$tmpl!./mobileslides",

	"jquery/ui/widget",
	"ep/ui/core",
	"ep/ui/slidebox"
], function( $, ep, tmplSlides ){

	var body,

		win = $(window),

		mobile,

		preventDefault = function( event ){
			event.preventDefault();
		};

	$.widget( "ui.mobileSlides", {
		options: {
			images: "img" // selector / array of data / data / image node / array of image nodes / jQuery object
		},

		_create: function(){
			var self = this,
				o = self.options,
				imgs = self.images = ep.ui.imgData(o.images)
					.on("change", $.proxy(self, "_change")),
				tmplData = {
					tBack		: "Back",
					tFullSize	: "Fullsize",
					images		: imgs
				};

			win.on("resize", $.proxy(self, "_resize"));

			self.element
				.on("click", $.proxy(self, "_open"));

			// load template and add element names to instance
			$.extend( self, tmplSlides([tmplData] ).appendTo("body").hide().tmplItem("elements") );

			self.back
				.on("click", $.proxy(self, "close"));

			self.fullsize
				.on("click", $.proxy(self, "_fullsize"));

			self.body
				.on("changeslide", function( event, o ){
					self.count.text( o.slide + " / " + o.slides );
					imgs.current( o.slide-1 );
				})
				.uiSlidebox({
					view: 1,
					arrow: true
				});

			self.foot
				.on("click", "li", function(){
					imgs.current( $(this).data("index") );
				})
				.uiSlidebox({
					view: 4,
					arrow: true
				});

			self.scrollTop = 0;

			self._setOptions(o);
		},

		_init: function(){
			var self = this;

			if( !body ){
				body = $("body");
			}

			if( !mobile ){
				mobile = $(".MobileLayout");
			}

			self._resize();
		},

		_fullsize: function(event){
			event.preventDefault();
			location.href = this.images.current().href;
		},

		_resize: function(){
			this.container.height( win.height() + ( /iPhone/.test(navigator.userAgent) ? 60 : 0) );
		},

		_change: function(){
			var self = this;

			self.body.uiSlidebox("goTo", self.images.currentItem + 1);
		},

		_open: function( event ){
			event.preventDefault();
			this.open();
			this._resize();
		},

		open: function(){
			var self = this;
			
			// save current scroll position to restore
			self.scrollTop = body.prop("scrollTop");

			body.prop("scrollTop",0)
				.on("scroll.mobileSlides", preventDefault);

			self.container.show();
			
			// Re init sizes
			self.body.uiSlidebox();
			self.foot.uiSlidebox();
			
			mobile.hide(); // Prevent mark of links in bottom layers in Android 2.x
		},

		close: function(){
			var self = this;

			body.prop("scrollTop",self.scrollTop)
				.off("scroll.mobileSlides", preventDefault);

			mobile.show(); // Prevent mark of links in bottom layers in Android 2.x
			self.container.hide();
		},

		destroy: function(){
			var self = this;

			self.images.off(".mobileSlides");
			self.container.remove();
			self._superApply(arguments);
		}
	});

	return ep;

});