define(['testsuite/qunit', 'jquery', 'ep/ui/slider', '$ready!'], function (QUnit, $, ep) {

	QUnit.module("ep/ui/slider");

	QUnit.asyncTest("ep/ui/slider", function () {
	
		// Check prototype methods.
		$.each(['_create', '_setOption', 'destroy'], function (i, fn) {
			QUnit.equal(
				$.isFunction($.ui.uiSlider.prototype[fn]),
				true,
				'$.isFunction(' + fn + '):'
			);
		});

		// Room for our test cases.
		var slider = [],
				instances = [];

		// Create *uiSlider* passing options in *data-attribute*.
		slider[1] = ep('#slider1').uiSlider();
		slider[2] = ep('#slider2').uiSlider();

		instances[1] = slider[1].data('uiUiSlider');
		instances[2] = slider[2].data('uiUiSlider');

		// Check label.
		$.each([1,2], function (i, j) {
			QUnit.equal(
				slider[j].find('.ep-uiSlider-label').text(),
				'Label' + j,
				'Label of *slider[' + j + ']*:'
			);
		});

		// Check value (and unit).
		QUnit.equal(
			slider[1].find('.ep-uiSlider-value').text(),
			'1' + ' px',
			'Value of *slider[' + 1 + ']*:'
		);
		QUnit.equal(
			slider[2].find('.ep-uiSlider-value').text(),
			'2',
			'Value of *slider[' + 2 + ']*:'
		);

		// Check info bubble.
		QUnit.equal(
			slider[1].find('img.BubbleHelp').length > 0,
			true,
			'*slider[' + 1 + ']*' + ' has info bubble:'
		);

		QUnit.equal(
			instances[1].infoNode.text(),
			'InfoText1',
			'*slider[' + 1 + ']*' + ' has info text:'
		);
		QUnit.equal(
			slider[2].find('img.BubbleHelp').length > 0,
			false,
			'*slider[' + 2 + ']*' + ' has info bubble:'
		);

		// Check min and max.
		QUnit.equal(
			instances[1]._valueMin(),
			1,
			'instances[' + 1 + ']' + '._valueMin():'
		);
		QUnit.equal(
			instances[2]._valueMin(),
			0,
			'instances[' + 2 + ']' + '._valueMin():'
		);
		QUnit.equal(
			instances[1]._valueMax(),
			5,
			'instances[' + 1 + ']' + '._valueMax():'
		);
		QUnit.equal(
			instances[2]._valueMax(),
			10,
			'instances[' + 2 + ']' + '._valueMax():'
		);

		// Check if sliding the slider changes the value.
		instances[1].element.trigger('slide', [{value : 2}]);
		QUnit.equal(
			slider[1].find('.ep-uiSlider-value').text(),
			'2' + ' px',
			'Value of *slider[' + 1 + ']* after sliding:'
		);
		instances[2].element.trigger('slide', [{value : 1}]);
		QUnit.equal(
			slider[2].find('.ep-uiSlider-value').text(),
			'1',
			'Value of *slider[' + 2 + ']* after sliding:'
		);


		// Destroy.
		instances[1].destroy();
		instances[2].destroy();

		// Check destroy.
		QUnit.equal(
			slider[1].children().length,
			0,
			'Children of *slider[' + 1 + ']*:'
		);
		QUnit.equal(
			slider[1].attr('class'),
			'',
			'Class attribute of *slider[' + 1 + ']*:'
		);
		QUnit.deepEqual(
			slider[1].data('epUiSlider'),
			$.parseJSON(slider[1].attr('data-ep-ui-slider')),
			'*instance[' + 1 + ']*' + 'successfully destroyed:'
		);
		QUnit.equal(
			slider[2].children().length,
			0,
			'Children of *slider[' + 2 + ']*:'
		);
		QUnit.equal(
			slider[2].attr('class'),
			'',
			'Class attribute of *slider[' + 2 + ']*:'
		);
		QUnit.deepEqual(
			slider[2].data('epUiSlider'),
			$.parseJSON(slider[2].attr('data-ep-ui-slider')),
			'*instance[' + 2 + ']*' + 'successfully destroyed:'
		);

		// Let's go, QUnit!
		QUnit.start();
	});
});