define(['testsuite/qunit', 'jquery', 'de_epages/mediagallery/filemanager', '$ready!'], function (QUnit, $, Filemanager) {

	QUnit.module("de_epages/mediagallery/filemanager");

	QUnit.asyncTest("de_epages/mediagallery/filemanager", function()	{

		// fake data for ajax requests
		var initArray = [{
				"currentGallerySize": "1344",
				"children": [{
					"files": [{
						"width": null,
						"extension": ".swf",
						"name": "promo.swf",
						"path": "/promo.swf",
						"height": null,
						"directory": "/MediaGallery",
						"modified": 1200994146,
						"size": 67089,
						"fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/promo.swf",
						"selectable": 0,
						"mimetype": "application/x-shockwave-flash"
					}, {
						"width": "240",
						"extension": ".gif",
						"name": "StartPagePromotion.gif",
						"path": "/StartPagePromotion.gif",
						"height": "113",
						"directory": "/MediaGallery",
						"modified": 1200994146,
						"size": 14453,
						"preview": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
						"fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
						"selectable": 1,
						"mimetype": "image/gif"
					}],
					"children": [{
						"files": [],
						"children": [],
						"metadata": {
							"displayname": "neu",
							"type": "MediaGallery",
							"path": "/neu"
						},
						"data": "neu",
						"state": null
					}],
					"metadata": {
						"displayname": "MediaGallery",
						"type": "MediaGallery",
						"path": ""
					},
					"data": "MediaGallery",
					"state": "closed"
				}, {
					"files": [],
					"children": [{
						"files": [],
						"children": [],
						"metadata": {
							"displayname": "Kunst, Kultur und Bildung",
							"type": "Header",
							"path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
						},
						"data": "ArtAndEducation",
						"state": null
					}, {
						"files": [],
						"children": [],
						"metadata": {
							"displayname": "Auto und Transport",
							"type": "Header",
							"path": "/StyleGroups/Branches/StyleGroups/Automotive"
						},
						"data": "Automotive",
						"state": null
					}],
					"metadata": {
						"displayname": "Kopfzeilenbilder",
						"type": "Header",
						"path": "/StyleGroups/Branches/StyleGroups"
					},
					"data": "HeaderImages",
					"state": "closed"
				}, {
					"files": [],
					"children": [{
						"files": [],
						"children": [],
						"metadata": {
							"displayname": "Kunst, Kultur und Bildung",
							"type": "Background",
							"path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
						},
						"data": "ArtAndEducation",
						"state": null
					}],
					"metadata": {
						"displayname": "Hintergrundbilder",
						"type": "Background",
						"path": "/StyleGroups/Branches/StyleGroups"
					},
					"data": "BackgroundImages",
					"state": "closed"
				}, {
					"files": [],
					"children": [{
						"files": [],
						"children": [],
						"metadata": {
							"displayname": "Gebäude",
							"type": "ImageDB",
							"path": "/ImageDBGroups/Buildings"
						},
						"data": "Buildings",
						"state": null
					}],
					"metadata": {
						"displayname": "{ImageDB}",
						"type": "ImageDB",
						"path": "ImageDBGroups"
					},
					"data": "ImageDB",
					"state": "closed"
				}],
				"maxGallerySize": "102400"
			}],
			directoryListArray = [{
				"files": [],
				"children": [],
				"metadata": {
					"displayname": "Kunst, Kultur und Bildung",
					"type": "Background",
					"path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
				},
				"data": "ArtAndEducation",
				"state": null
			}],
			createDirectoryArray = [{
				"currentGallerySize": "3143",
				"children": [{
					"files": [],
					"children": [],
					"metadata": {
						"type": "MediaGallery",
						"path": "/Neuen_Ordner_anlegen"
					},
					"data": "Neuen_Ordner_anlegen",
					"state": null
				}]
			}],
			renameDirectoryObj = {
				"ResultingPath": "/newName",
				"ResultingDirectory": "newName"
			},
			renameFileObj = {
				"ResultingPath": "//StartPagePromotion.gif",
				"ResultingFilename": "StartPagePromotion.gif",
				"ResultingFullPath": "/WebRoot/Store/Shops/DemoShop/MediaGallery///StartPagePromotion.gif"
			},
			deleteFolderObj = {
				"currentGallerySize": "3165"
			},
			deleteFileObj = {
				"currentGallerySize": "1180"
			},

			selectable,
			filemanager,
			e,

			// helper method to trigger select on a selectable
			selectSelectableElement = function (selectableContainer, elementToSelect) {
				elementToSelect.addClass("ui-selecting");

				selectableContainer
					.data("selectable")
					._mouseStop(null);
			};

		// fake ep.ajax for the different requests
		$.ajax = function( o ){
			var result = $.Deferred();

			if(o.data.ViewAction === "JSONInitMediaGallery"){
				result.resolve(initArray);
			}
			if(o.data.ViewAction === "JSONGetDirectoryList"){
				result.resolve(directoryListArray);
			}

			switch(o.data.ChangeAction){
				case "JSONCreateDirectory":
					result.resolve(createDirectoryArray);
					break;
				case "JSONRenameDirectory":
					result.resolve(renameDirectoryObj);
					break;
				case "JSONRenameFile":
					result.resolve(renameFileObj);
					break;
				case "JSONDeleteDirectory":
					result.resolve(deleteFolderObj);
					break;
				case "JSONDeleteFile":
					result.resolve(deleteFileObj);
					break;
			}

			return result;
		};

		// create file manager
		filemanager = Filemanager.init({currentFolder: "MediaGallery"});

		QUnit.equal(
			filemanager.$el.length,
			1,
			"Filemanager was successfully created"
		);

		// get current gallery size
		QUnit.equal(
			filemanager.getCurrentUsage(),
			"1344",
			"Get the current gallery size"
		);

		// create new folder
		$(".ep-mediagallery-newFolderButton").trigger("click");

		QUnit.equal(
			$(".ep-mediagallery-folderContainer").length,
			2,
			"New folder was created - now there are 2 folders"
		);

		$(".changeOrder").trigger("click");

		// select all files from "Your files"
		$(".ep-mediagallery-fileContainer")
			.addClass("ui-selected")
			.trigger("click");

		QUnit.equal(
			filemanager.getSelectedElements().length,
			1,
			"Get number of selected files (only selectable ones)"
		);

		// create event
		e = $.Event("keypress");
		e.which = 13;

		// change name from first folder to "newName"
		$(".ep-mediagallery-itemName:first").trigger("click");
		$(".ep-mediagallery-folderContainer input")
			.val("newName")
			.trigger(e);

		QUnit.equal(
			$(".ep-mediagallery-folderContainer:first").find(".ep-mediagallery-itemName").html(),
			"newName",
			"Get the name of the first folder which was renamed"
		);

		$(".ep-mediagallery-fileContainer:first").find(".ep-mediagallery-itemName").trigger("click");
		$(".ep-mediagallery-fileContainer input")
			.val("StartPagePromotion")
			.trigger(e);

		QUnit.equal(
			$(".ep-mediagallery-fileContainer:first").find(".ep-mediagallery-itemName").html(),
			"StartPagePromotion.gif",
			"Get the name of the first file which was renamed"
		);

		// open HeaderImages tree
		$($(".ep-mediagallery-treeHead")[1]).trigger("click");
		$(".jstree-leaf:visible").find("a").trigger("click");

		QUnit.equal(
			$(".ep-mediagallery-listViewHeader").find("h1").html(),
			"Kunst, Kultur und Bildung",
			"Get the name of the currently opened folder"
		);

		window.setTimeout(function(){
			// open Your files tree
			$(".ep-mediagallery-treeHead:first").trigger("click");

			QUnit.equal(
				$(".ep-mediagallery-treeHead").hasClass("active"),
				true,
				"The first tab of the accordion was selected"
			);

			selectable = $(".ep-mediagallery-itemList");

			// select 2 items and remove them
/*
			selectSelectableElement($(".ep-mediagallery-itemList"), $(".ep-mediagallery-itemList").children(":even"));
			$(".ep-mediagallery-deleteItem").trigger("click");
			$(".ui-dialog-buttonset").find(".ep-uiInput-button:first").trigger("click");
*/

			QUnit.equal(
				$(".ep-mediagallery-itemList li").length,
				2,
				"Get the number of items after 2 were removed"
			);

			QUnit.start();

		}, 500);
	});
});