use strict;
use Test::More tests => 5;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::MultiStoreCommon::API::Constants;
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::WebService::API::Client;

local $DE_EPAGES::MultiStoreCommon::API::Constants::IS_MULTISTORE_EXIMPORT = 1;
local $ENV{'EPAGES_TESTSITE'} = 'MultiStore' unless defined $ENV{'EPAGES_TESTSITE'};

my $SUBSHOPALIAS = 'TestSubShop';
my $ROOTPATH = "/Shops/$SUBSHOPALIAS";

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $MSService = GetWebServiceClient();
    my $dbi = GetCurrentDBHandle();
    my $Shop = LoadObjectByPath($ROOTPATH);
    my $CategoryPath = 'Categories/TestJackets';
    my $CategoryPath2 = 'Categories/TestJackets2';

    #Test updateSubShopObjects
    my $ahUpdates = [
        { 'ObjectPath' => $CategoryPath, 'Remove' => 0 },
        { 'ObjectPath' => $CategoryPath2, 'Remove' => 1 },
    ];

    $MSService->updateSubShopObjects($SUBSHOPALIAS,$ahUpdates);

    my $dbResult = $dbi->execute(
        'SELECT objectpath, remove FROM objectupdate WHERE shopid=?',
        [ $Shop->id ], ['String', 'Integer']
    );

    my @hResults =
        grep {
               ( $_->{'ObjectPath'} eq $CategoryPath and $_->{'Remove'} == 0 )
            or ( $_->{'ObjectPath'} eq $CategoryPath2 and $_->{'Remove'} == 1 )
        } map {
            { 'ObjectPath' => $_->[0], 'Remove' => $_->[1], }
        } @$dbResult;

    ok( scalar @hResults == 2, 'updateSubShopObjects' );

    #Test deleteSubShopObjects
    my $SubShopObjectPath = "/Shops/$SUBSHOPALIAS/$CategoryPath";
    my $SubShopObjectPath2 = "/Shops/$SUBSHOPALIAS/$CategoryPath2";
    ok( ExistsObjectByPath($SubShopObjectPath) && ExistsObjectByPath($SubShopObjectPath2), 'exist SubShopObjects' );

    $MSService->deleteSubShopObjects($SUBSHOPALIAS);

    $DE_EPAGES::DataCache::API::Cache::CACHE->resetConnection($dbi->cacheKey);
    ok( !ExistsObjectByPath($SubShopObjectPath) && !ExistsObjectByPath($SubShopObjectPath2), 'deleteSubShopObjects' );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub GetWebServiceClient {
    return DE_EPAGES::WebService::API::Client
            ->uri( 'urn://epages.de/WebService/MultiStoreService/2013/04' )
            ->proxy( 'http://aploeger/epages/MultiStore.soap' )
            ->userinfo( '/Users/wsadmin:wsadmin' )
            ->xmlschema('2001');
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};

    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    SynchronizeCache();
    return 1;
}

sub Init {

    die $ENV{'EPAGES_TESTSITE'} . ': Cartridge MultiStore is not installed.'
        unless ExistsCartridgeByPackage('DE_EPAGES::MultiStore');

    Cleanup();
    ImportObjects('SubShopEmpty.xml','/Shops');
    ImportObjects('CategoryJackets.xml',"/Shops/$SUBSHOPALIAS/Categories");
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
