use strict;
use Test::Simple tests => 22;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectToSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias );

sub ActionByName {
    my ($Class, $Name) = @_;
    my ($Action) = grep {$_->alias eq $Name} @{$Class->get('Actions')};
    return $Action;
}
sub RoleByName {
    my ($Class, $Name) = @_;
    my ($Role) = grep {$_->alias eq $Name} @{$Class->get('Roles')};
    return $Role;
}

sub ExistsDirectAction {
    my ($Class, $Name) = @_;
    my ($Action) = grep {$_->alias eq $Name} @{$Class->get('DirectActions')};
    return (defined $Action ? 1 : 0);
}

sub ExistsDirectRole {
    my ($Class, $Name) = @_;
    my ($Role) = grep {$_->alias eq $Name} @{$Class->get('DirectRoles')};
    return (defined $Role ? 1 : 0);
}

sub Main {
    ConnectToSite();

    ok( Init(), 'initialization' );

    my $ClassObject = LoadClassByAlias( 'Object' );

    my $Class1 = $ClassObject->insertSubClass( { Alias => 'C1' } );

    my $Class2 = $Class1->insertSubClass( { Alias => 'C2' } );

    ok( ExistsClassByAlias( 'C2' ), 'existsName' );

    my $ActionView = $Class1->folder('Actions')->insertChild('Action', { Alias => 'C1View', ActionClass=> $Class1, 'Package'=>'main' } );
    ok( $ActionView->id, 'insertAction' );
    ok( ActionByName($Class1,'C1View')->id == $ActionView->id, 'action found' );

    my $ActionEdit = $Class2->folder('Actions')->insertChild('Action', { Alias => 'C1Edit', ActionClass=> $Class2, 'Package'=>'main'  } );
    ok( $ActionEdit->alias eq 'C1Edit', 'action name' );
    ok( ActionByName($Class2,'C1View')->id == $ActionView->id, 'inherited action' );

    my $ObjectActionsCount = scalar @{ $ClassObject->get('Actions') };
    ok( 2 + $ObjectActionsCount == scalar @{ $Class2->get('Actions') }, 'actions count' );
    ok( ExistsDirectAction($Class2, 'C1Edit' ), 'existsDirectAction 1' );
    ok( !ExistsDirectAction($Class2,'C1View' ), 'existsDirectAction 1' );

    my $RoleManager = $Class1->folder('Roles')->insertChild('Role', { Alias => 'Manager', ActionClass=> $Class1 } );
    ok( $RoleManager->id, 'insertRole' );
    ok( 1 == scalar @{ $Class2->get('Roles') }, 'roles count' );
    ok( 2 + $ObjectActionsCount == scalar @{ $Class2->get('Actions') }, 'actions count is not incremented' );
    ok( ExistsDirectRole($Class1, 'Manager' ), 'existsDirectRole 1' );
    ok( !ExistsDirectRole($Class2,'Manager' ), 'existsDirectRole 2' );

    ok( 0 == scalar @{ $RoleManager->actions }, 'before addAction' );
    $RoleManager->addAction( $ActionView );
    $RoleManager->addAction( $ActionEdit );
    ok( 2 == scalar @{ $RoleManager->actions }, 'after addAction' );

    ok( 1 == scalar @{ $ActionEdit->roles }, 'action roles count' );
    ok( $ActionEdit->roles->[0]->id == $RoleManager->id, 'action role' );

    ok( $RoleManager->existsAction( $ActionEdit ), 'before removeAction' );
    $RoleManager->removeAction( $ActionEdit );
    ok( !$RoleManager->existsAction( $ActionEdit ), 'after removeAction' );

    ok( RoleByName($Class2, 'Manager' )->alias eq 'Manager', 'inherited role' );

    ok( Cleanup(), 'cleanup' );

    DisconnectSite();
}

sub Cleanup {
    foreach my $ClassName ( qw( C1 C2 C3 C4 ) ) {
        next unless ExistsClassByAlias( $ClassName );
        LoadClassByAlias( $ClassName )->delete;
    }

    return 1;
}

sub Init {
    Cleanup();

    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
