use Test::Simple tests => 4;
use strict;

use DE_EPAGES::Core::API::File qw ( DeleteFile );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Object::API::XMLCompare qw ( CompareXML );
use DE_EPAGES::Object::API::XMLStructureImportHandler;
use DE_EPAGES::Object::API::XMLStructureExportDriver;

use Readonly;
Readonly::Scalar my $EXPORT_FILENAME => 'Temp.xml';

sub TestMemberOfImportExport {
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $ExportDriver  = DE_EPAGES::Object::API::XMLStructureExportDriver->new();

    my $XML = 'DefaultShop.xml';
    my $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($EXPORT_FILENAME, $XMLStruct);
    ok(
        (
            CompareXML($XML, $EXPORT_FILENAME, {'printmessage' => '1'}) and
            CompareXML($EXPORT_FILENAME, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    return;
}

sub TestAttributePermissionImportExport {
    my $ImportHandler = DE_EPAGES::Object::API::XMLStructureImportHandler->new();
    my $ExportDriver  = DE_EPAGES::Object::API::XMLStructureExportDriver->new();

    my $XML = 'Permissions.xml';
    my $XMLStruct = $ImportHandler->getXMLStructure($XML);
    $ExportDriver->exportXML($EXPORT_FILENAME, $XMLStruct);
    ok(
        (
            CompareXML($XML, $EXPORT_FILENAME, {'printmessage' => '1'}) and
            CompareXML($EXPORT_FILENAME, $XML, {'printmessage' => '1'})
        )
        , "$XML export and original xml are equal"
    );

    return;
}

sub Init {
    CleanUp();

    return 1;
}

sub CleanUp {
    DeleteFile($EXPORT_FILENAME);

    return 1;
}

RunScript(
    Debug => 1,
    Sub   => sub {
        ok(Init(), 'initialization done');

        TestMemberOfImportExport();
        TestAttributePermissionImportExport();

        ok( CleanUp(), 'cleanup done');
    },
);

1;