define(['testsuite/qunit', 'jquery', 'de_epages/presentation/ui/previeweditor', '$ready!'], function (QUnit, $, de_epages) {

	QUnit.module("de_epages/presentation/ui/previeweditor");

	QUnit.asyncTest("de_epages/presentation/ui/previeweditor", function () {

		top.jq = jq = $;
		top.jQuery = top.$ = jQuery;

		// Check prototype methods.
		$.each(['_create', '_setOption', 'destroy', '_execEpAlmostFullscreen', '_afterSetData'], function (i, fn) {

			QUnit.equal(
				$.isFunction($.ui.presentationUiPrevieweditor.prototype[fn]),
				true,
				'$.isFunction(' + fn + '):'
			);

		});

		var $headerDiv = $('#EmailHeader').presentationUiPrevieweditor();
		var $signatureDiv = $('#EmailSignature').presentationUiPrevieweditor();
		var $headerTextarea = $('textarea[name="Header"]');
		var $signatureTextarea = $('textarea[name="Signature"]');


		// Check for *click* handler.
		QUnit.equal(
			$headerDiv.data('events') && $headerDiv.data('events').click && ($headerDiv.data('events').click.length === 1),
			true,
			'*$headerDiv* has *click* handler.'
		);

		QUnit.equal(
			$signatureDiv.data('events') && $signatureDiv.data('events').click && ($signatureDiv.data('events').click.length === 1),
			true,
			'*$signatureDiv* has *click* handler.'
		);

		var headerInstance = $($headerDiv).data('uiPresentationUiPrevieweditor');
		var signatureInstance = $($signatureDiv).data('uiPresentationUiPrevieweditor');

		// Check data of *ckeditor* instances.
		QUnit.equal(
			headerInstance.ckeditor.getData(),
			$headerTextarea.val(),
			'headerInstance.ckeditor.getData()'
		);

		QUnit.equal(
			signatureInstance.ckeditor.getData(),
			$signatureTextarea.val(),
			'signatureInstance.ckeditor.getData()'
		);

		setTimeout(function () {

			// Open *almostFullscreen*.
			$headerDiv.children().first().trigger('click');

			setTimeout(function () {

				// *SetData*. (Closing AlmostFullscreen would trigger this too.)
				headerInstance.ckeditor.setData('AfterSetData method works.')

				setTimeout(function () {

					QUnit.equal(
						headerInstance.ckeditor.getData(),
						$headerTextarea.val(),
						'headerInstance.ckeditor.getData()'
					);

					QUnit.equal(
						$headerTextarea.val(),
						'AfterSetData method works.',
						'headerInstance.ckeditor.getData()'
					);

					QUnit.equal(
						$headerDiv.get(0).innerHTML,
						'AfterSetData method works.',
						'$headerDiv.get(0).innerHTML'
					);

					// Destroy.
					headerInstance.destroy();
					signatureInstance.destroy();

					QUnit.equal(
						$headerDiv.data('events') && $headerDiv.data('events').click,
						undefined,
						'*$headerDiv* has *click* handler.'
					);
					QUnit.equal(
						$signatureDiv.data('events') && $signatureDiv.data('events').click,
						undefined,
						'*$signatureDiv* has *click* handler.'
					);

					// Check data of *ckeditor* instances.
					QUnit.deepEqual(
						$($headerDiv).data('de_epagesPresentationUiPrevieweditor'),
						$.parseJSON($headerDiv.attr('data-de_epages-presentation-ui-previeweditor')),
						"$($headerDiv).data('de_epagesPresentationUiPrevieweditor')"
					);

					QUnit.deepEqual(
						$($signatureDiv).data('de_epagesPresentationUiPrevieweditor'),
						$.parseJSON($signatureDiv.attr('data-de_epages-presentation-ui-previeweditor')),
						"$($signatureDiv).data('de_epagesPresentationUiPrevieweditor')"
					);

					// Let's go, QUnit!
					QUnit.start();

				}, 500);
			}, 500);
		}, 500);
	});
});